using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

using XNACS1Lib;

namespace ParticleStuff
{
    public class Game1 : XNACS1Base
    {
        XNACS1Circle circleWithTail;
        XNACS1ParticleEmitter.TailEmitter controllableTail;
        XNACS1Circle tailHead;
        XNACS1ParticleEmitter.ExplodeEmitter controllableExplosion;
        XNACS1ParticleEmitter.ExplodeEmitter constantExplosion;
        VortexEmitter constantVortex;
        HaloEmitter constantHalo;
        XNACS1ParticleEmitter.FireEmitter constantFire;
        PaintSplatterEmitter controlPaintSplatter;
        LightningEmitter controllLighting;
        XNACS1ParticleEmitter.TailEmitter tail;
        double i = 0;

        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        static void Main(string[] args)
        {
            using (Game1 game = new Game1())
            {
                game.Run();
            }
        }

        protected override void InitializeWorld()
        {
            World.SetWorldCoordinate(Vector2.One, 100);
            World.SetWorldCoordinate(new Vector2(-World.WorldDimension.X / 2, -World.WorldDimension.Y / 2), 100);
            World.SetBackgroundColor(Color.Black);

            //controllabletail

            controllableTail = new XNACS1ParticleEmitter.TailEmitter(Vector2.Zero, 50, .5f, "Particle009", Color.Green);
            tailHead = new XNACS1Circle(new Vector2(35, 20), .6f, "Particle009");
            tailHead.TextureTintColor = Color.Green;
            controllableTail.AttachedPrimitive = tailHead;
            controllableTail.TailTravelMode = ParticleTravelMode.CompressionSine;

            //explosion following tail (a button to explode)
            controllableExplosion = new XNACS1ParticleEmitter.ExplodeEmitter(Vector2.Zero, 15, .4f, "Particle011", Color.OrangeRed, 3);
            controllableExplosion.AttachedPrimitive = controllableTail;

            //constant explosion
            constantExplosion = new XNACS1ParticleEmitter.ExplodeEmitter(new Vector2(-15, -15), 15, .4f, "Particle011", Color.OrangeRed, 3);

            //constant vortex
            constantVortex = new VortexEmitter(new Vector2(-15, 15), 30, .4f, "Particle011", Color.Aquamarine, 5,
                ParticleTravelMode.TransverseSine, .5f, .3f, VortexEmitter.VortexFadeMode.FadeInOut, true);
            constantVortex.ParticlesPerEmit = 10;

            //constant Halo
            constantHalo = new HaloEmitter(new Vector2(-35, 20), 15, .4f, "Particle008", Color.Green, 3);

            //constant fire
            constantFire = new XNACS1ParticleEmitter.FireEmitter(new Vector2(-35, -5), 30, .5f, "Particle008", Color.OrangeRed, 20, 3);
            constantFire.ParticlesPerEmit = 5;

            //controll paint splatter
            controlPaintSplatter = new PaintSplatterEmitter(new Vector2(0, 0), 300, .5f, "Particle011", Color.White, new Vector2(-35, -20));

            //controll lightning
            controllLighting = new LightningEmitter(new Vector2(0, 0), 40, .3f, .3f, "Particle011", Color.YellowGreen);

            //circle on path with tail
            tail = new XNACS1ParticleEmitter.TailEmitter(Vector2.One, 30, .5f, "Particle011", Color.LightBlue);
            circleWithTail = new XNACS1Circle(Vector2.Zero, 1);
            circleWithTail.CenterX = 8 * (float)Math.Sin(i);
            circleWithTail.CenterY = 8 * (float)Math.Cos(i);
            tail.AttachedPrimitive = circleWithTail;
            circleWithTail.Color = Color.Red;
            tail.ShouldTravel = true;
            tail.TravelMode = ParticleTravelMode.TransverseSine;
            tail.DisplacementAmplitude = 5;
            tail.DisplacementFrequency = .2f;


            XNACS1Particle red = new XNACS1Particle(new Vector2(16, 16), 7, 0, "Particle011");
            red.TextureTintColor = new Color(255, 0, 0);
            XNACS1Particle green = new XNACS1Particle(new Vector2(21, 16), 7, 0, "Particle011");
            green.TextureTintColor = new Color(0, 255, 0);
            XNACS1Particle blue = new XNACS1Particle(new Vector2(19, 21), 7, 0, "Particle011");
            blue.TextureTintColor = new Color(0, 0, 255);
        }

        protected override void UpdateWorld()
        {
            if (GamePad.ButtonBackClicked())
                Exit();

            i += .05;

            circleWithTail.CenterX = 8 * (float)Math.Sin(i);
            circleWithTail.CenterY = 8 * (float)Math.Cos(i);

            tailHead.Center += GamePad.ThumbSticks.Right;

            if (GamePad.ButtonAClicked())
            {
                controllableExplosion.Explode(100);
            }
            if (GamePad.ButtonStartClicked())
            {
                controlPaintSplatter.Splatter(10000);
            }
            constantExplosion.Explode(20);
            constantHalo.DrawHalo(30);

            if (GamePad.ButtonBClicked())
            {
                controllLighting.Bolt(controllableTail.Center, 5, 3);
                if (constantVortex.TravelDirection)
                    constantVortex.TravelDirection = false;
                else
                    constantVortex.TravelDirection = true;
            }
            if (GamePad.ButtonXClicked())
            {
                if (constantVortex.FadeMode == VortexEmitter.VortexFadeMode.FadeInOut)
                    constantVortex.FadeMode = VortexEmitter.VortexFadeMode.FadeIn;
                else if (constantVortex.FadeMode == VortexEmitter.VortexFadeMode.FadeIn)
                    constantVortex.FadeMode = VortexEmitter.VortexFadeMode.FadeOut;
                else
                    constantVortex.FadeMode = VortexEmitter.VortexFadeMode.FadeInOut;
            }
            if (GamePad.ButtonYClicked())
            {
                if (constantVortex.ParticleTravelMode == ParticleTravelMode.TransverseSine)
                    constantVortex.ParticleTravelMode = ParticleTravelMode.CompressionSine;
                else if (constantVortex.ParticleTravelMode == ParticleTravelMode.CompressionSine)
                    constantVortex.ParticleTravelMode = ParticleTravelMode.HorizontalSine;
                else if (constantVortex.ParticleTravelMode == ParticleTravelMode.HorizontalSine)
                    constantVortex.ParticleTravelMode = ParticleTravelMode.VerticalSine;
                else if (constantVortex.ParticleTravelMode == ParticleTravelMode.VerticalSine)
                    constantVortex.ParticleTravelMode = ParticleTravelMode.RandomDisplace;
                else if (constantVortex.ParticleTravelMode == ParticleTravelMode.RandomDisplace)
                    constantVortex.ParticleTravelMode = ParticleTravelMode.Linear;
                else
                    constantVortex.ParticleTravelMode = ParticleTravelMode.TransverseSine;

                if (tail.TravelMode == ParticleTravelMode.TransverseSine)
                    tail.TravelMode = ParticleTravelMode.CompressionSine;
                else if (tail.TravelMode == ParticleTravelMode.CompressionSine)
                    tail.TravelMode = ParticleTravelMode.HorizontalSine;
                else if (tail.TravelMode == ParticleTravelMode.HorizontalSine)
                    tail.TravelMode = ParticleTravelMode.VerticalSine;
                else if (tail.TravelMode == ParticleTravelMode.VerticalSine)
                    tail.TravelMode = ParticleTravelMode.RandomDisplace;
                else if (tail.TravelMode == ParticleTravelMode.RandomDisplace)
                    tail.TravelMode = ParticleTravelMode.Linear;
                else
                    tail.TravelMode = ParticleTravelMode.TransverseSine;
            }
        }
    }
}
