﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

using XNACS1Lib;

namespace ParticleStuff
{
    /// <summary>
    /// Emits a Halo when drawHalo() is called
    /// </summary>
    public class HaloEmitter : XNACS1ParticleEmitter
    {
        float m_Radius;
        float m_Speed;

        /// <summary>
        /// Constructor for Halo emitter
        /// </summary>
        /// <param name="center"></param>
        /// <param name="initialParticleLife"></param>
        /// <param name="particleSize"></param>
        /// <param name="tex"></param>
        /// <param name="defaultColor"></param>
        /// <param name="radius">radius? of the Odd Emitter</param>
        public HaloEmitter(Vector2 center, int initialParticleLife, float particleSize, string tex, Color defaultColor, float radius)
            : base(center, initialParticleLife, particleSize, tex, defaultColor)
        {
            AutoEmitEnabled = false;
            m_Radius = radius;
            m_Speed = radius / initialParticleLife;
        }

        /// <summary>
        /// Get or set the speed at which particles move
        /// Default is radius / life;
        /// </summary>
        public float HaloSpeed
        {
            get { return m_Speed; }
            set { m_Speed = value; }
        }

        /// <summary>
        /// does an emit for the number of particles
        /// </summary>
        /// <param name="numParticles"></param>
        public void DrawHalo(int numParticles)
        {
            for (int i = 0; i < numParticles; i++)
            {
                Emit();
            }
        }

        /// <summary>
        /// Overloaded NewParticle Method, gives Halo particles initial velocity and sets
        /// ShouldTravel = true.
        /// </summary>
        /// <param name="newParticle"></param>
        protected override void NewParticle(XNACS1Particle newParticle)
        {
            float theta = XNACS1Base.RandomFloat(0, (float)(2 * Math.PI));
            newParticle.CenterX += m_Radius * (float)Math.Cos(theta);
            newParticle.CenterY += m_Radius * (float)Math.Sin(theta);
            Vector2 randomVelocity = new Vector2(XNACS1Base.RandomFloat(-1, 1), XNACS1Base.RandomFloat(-1, 1));
            randomVelocity *= m_Speed;
            newParticle.Velocity = randomVelocity;
            newParticle.ShouldTravel = true;
        }

        /// <summary>
        /// Overloaded UpdateParticle method, makes particles fade out according to what
        /// percentage of the default life they have remaining.
        /// </summary>
        /// <param name="particle"></param>
        protected override void UpdateParticle(XNACS1Particle particle)
        {
            int temp = (255 * particle.Life) / InitialLife;
            if (temp > 255) temp = 255;
            else if (temp < 0) temp = 0;
            Color tempColor = particle.TextureTintColor;
            tempColor.A = (byte)temp;
            particle.TextureTintColor = tempColor;
        }
    }
}
