﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

using XNACS1Lib;

namespace ParticleStuff
{
    /// <summary>
    /// Emits a splatter of paint
    /// </summary>
    public class PaintSplatterEmitter : XNACS1ParticleEmitter
    {
        Vector2 m_EndPos;
        float m_TotalDistance;
        float m_Distance;
        Vector2 m_Direction;
        Vector2 m_CurrentPos;

        /// <summary>
        /// cosntructor for paint splatter emitter
        /// </summary>
        /// <param name="center"></param>
        /// <param name="initialParticleLife"></param>
        /// <param name="particleSize"></param>
        /// <param name="tex"></param>
        /// <param name="defaultColor"></param>
        /// <param name="endPos">end position</param>
        public PaintSplatterEmitter(Vector2 center, int initialParticleLife, float particleSize, string tex, Color defaultColor, Vector2 endPos)
            : base(center, initialParticleLife, particleSize, tex, defaultColor)
        {
            AutoEmitEnabled = false;
            m_EndPos = endPos;
            m_Direction = endPos - center;
            m_TotalDistance = m_Direction.Length();
            m_Direction.Normalize();
            m_CurrentPos = center;
        }

        /// <summary>
        /// Creates particles which bolt outward from the center of the emitter to the end pos.
        /// </summary>
        /// <param name="NumParticles">Number of particles to create.</param>
        public void Splatter(int NumParticles)
        {
            m_Distance = m_TotalDistance / NumParticles;
            m_CurrentPos = Center;
            for (int i = 0; i < NumParticles; i++)
            {
                Emit();
            }
        }

        /// <summary>
        /// moves along in a random direction splattering paint
        /// </summary>
        /// <param name="newParticle"></param>
        protected override void NewParticle(XNACS1Particle newParticle)
        {
            newParticle.Center = (m_CurrentPos + (m_Direction * m_Distance)) + new Vector2(XNACS1Base.RandomFloat(-1, 1), XNACS1Base.RandomFloat(-1, 1));
            m_CurrentPos = newParticle.Center;
        }

        /// <summary>
        /// Overloaded UpdateParticle method, makes particles fade out according to what
        /// percentage of the default life they have remaining.
        /// </summary>
        /// <param name="particle"></param>
        protected override void UpdateParticle(XNACS1Particle particle)
        {
            int temp = (255 * particle.Life) / InitialLife;
            if (temp > 255) temp = 255;
            else if (temp < 0) temp = 0;
            Color tempColor = particle.TextureTintColor;
            tempColor.A = (byte)temp;
            particle.TextureTintColor = tempColor;
        }
    }
}
